@@ -14,7 +14,6 @@ use crate::frame::{Frame, RawFrame};
 use crate::geom;
 use crate::state;
 use crate::time::DurationF64;
-use crate::ui;
 use crate::wgpu;
 use crate::window::{self, Window};
 use find_folder;
@@ -26,7 +25,6 @@ use std::path::PathBuf;
 use std::sync::atomic::{self, AtomicBool};
 use std::sync::Arc;
 use std::time::Duration;
-use wgpu_upstream::SwapChain;
 use winit;
 use winit::event_loop::ControlFlow;
 
@@ -59,6 +57,7 @@ enum View<Model = ()> {
 /// A nannou `App` builder.
 pub struct Builder<M = (), E = Event> {
     model: ModelFn<M>,
+    config: Config,
     event: Option<EventFn<M, E>>,
     update: Option<UpdateFn<M>>,
     default_view: Option<View<M>>,
@@ -67,7 +66,7 @@ pub struct Builder<M = (), E = Event> {
     default_window_size: Option<DefaultWindowSize>,
     capture_frame_timeout: Option<Option<Duration>>,
     max_capture_frame_jobs: Option<u32>,
-    backends: wgpu::BackendBit,
+    backends: wgpu::Backends,
 }
 
 /// A nannou `Sketch` builder.
@@ -110,13 +109,12 @@ pub struct App {
     pub(crate) event_loop_proxy: Proxy,
     pub(crate) windows: RefCell<HashMap<window::Id, Window>>,
     /// The wgpu backends to choose between.
-    backends: wgpu::BackendBit,
+    backends: wgpu::Backends,
     /// The main wgpu instance.
     instance: wgpu::Instance,
     /// A map of active wgpu physical device adapters.
     adapters: wgpu::AdapterMap,
     draw_state: DrawState,
-    pub(crate) ui: ui::Arrangement,
     /// The window that is currently in focus.
     pub(crate) focused_window: RefCell<Option<window::Id>>,
     /// The current state of the `Mouse`.
@@ -192,12 +190,13 @@ struct LoopState {
     loop_start: Instant,
     last_update: Instant,
     total_updates: u64,
+    events_since_wakeup: usize,
 }
 
 /// The mode in which the **App** is currently running the event loop and emitting `Update` events.
 #[derive(Clone, Debug, PartialEq)]
 pub enum LoopMode {
-    /// Synchronises `Update` events with requests for a new image by the swap chain.
+    /// Synchronises `Update` events with requests for a new frame for the surface.
     ///
     /// The result of using this loop mode is similar to using vsync in traditional applications.
     /// E.g. if you have one window running on a monitor with a 60hz refresh rate, your update will
@@ -239,7 +238,7 @@ where
     M: 'static,
 {
     /// The default set of backends requested.
-    pub const DEFAULT_BACKENDS: wgpu::BackendBit = wgpu::DEFAULT_BACKENDS;
+    pub const DEFAULT_BACKENDS: wgpu::Backends = wgpu::DEFAULT_BACKENDS;
 
     /// Begin building the `App`.
     ///
@@ -255,6 +254,7 @@ where
     pub fn new(model: ModelFn<M>) -> Self {
         Builder {
             model,
+            config: Config::default(),
             event: None,
             update: None,
             default_view: None,
@@ -281,6 +281,7 @@ where
     {
         let Builder {
             model,
+            config,
             update,
             default_view,
             exit,
@@ -293,6 +294,7 @@ where
         } = self;
         Builder {
             model,
+            config,
             event: Some(event),
             update,
             default_view,
@@ -385,6 +387,12 @@ where
         self
     }
 
+    /// Specify the default initial loop mode for this app.
+    pub fn loop_mode(mut self, mode: LoopMode) -> Self {
+        self.config.loop_mode = mode;
+        self
+    }
+
     /// The maximum number of simultaneous capture frame jobs that can be run per window before we
     /// block and wait for the existing jobs to complete.
     ///
@@ -423,8 +431,8 @@ where
 
     /// Specify the set of preferred WGPU backends.
     ///
-    /// By default, this is `wgpu::BackendBit::PRIMARY`.
-    pub fn backends(mut self, backends: wgpu::BackendBit) -> Self {
+    /// By default, this is `wgpu::Backends::PRIMARY`.
+    pub fn backends(mut self, backends: wgpu::Backends) -> Self {
         self.backends = backends;
         self
     }
@@ -457,6 +465,7 @@ where
             .unwrap_or(Some(Self::DEFAULT_CAPTURE_FRAME_TIMEOUT));
         let event_loop_window_target = Some(EventLoopWindowTarget::Owned(event_loop));
         let app = App::new(
+            self.config,
             event_loop_proxy,
             event_loop_window_target,
             self.default_window_size,
@@ -499,6 +508,14 @@ impl<E> SketchBuilder<E>
 where
     E: LoopEvent,
 {
+    /// Specify the default initial loop mode for this sketch.
+    ///
+    /// This method delegates to `Builder::loop_mode`.
+    pub fn loop_mode(mut self, mode: LoopMode) -> Self {
+        self.builder = self.builder.loop_mode(mode);
+        self
+    }
+
     /// The size of the sketch window.
     pub fn size(mut self, width: u32, height: u32) -> Self {
         self.builder = self.builder.size(width, height);
@@ -601,22 +618,22 @@ impl App {
 
     // Create a new `App`.
     fn new(
+        config: Config,
         event_loop_proxy: Proxy,
         event_loop_window_target: Option<EventLoopWindowTarget>,
         default_window_size: Option<DefaultWindowSize>,
         max_capture_frame_jobs: u32,
         capture_frame_timeout: Option<Duration>,
-        backends: wgpu::BackendBit,
+        backends: wgpu::Backends,
     ) -> Self {
         let instance = wgpu::Instance::new(backends);
         let adapters = Default::default();
         let windows = RefCell::new(HashMap::new());
         let draw = RefCell::new(draw::Draw::default());
-        let config = RefCell::new(Default::default());
+        let config = RefCell::new(config);
         let renderers = RefCell::new(Default::default());
         let draw_state = DrawState { draw, renderers };
         let focused_window = RefCell::new(None);
-        let ui = ui::Arrangement::new();
         let mouse = state::Mouse::new();
         let keys = state::Keys::default();
         let duration = state::Time::default();
@@ -634,7 +651,6 @@ impl App {
             windows,
             config,
             draw_state,
-            ui,
             mouse,
             keys,
             duration,
@@ -744,27 +760,6 @@ impl App {
         windows.keys().cloned().collect()
     }
 
-    /// Return the **SwapChain** for the given `window_id`, rebuilding if it doesn't exist.
-    ///
-    /// **Panics** if `window_id` does not exist, or if the swap chain is missing and cannot be rebuilt.
-    fn take_or_rebuild_swap_chain(&self, window_id: window::Id) -> SwapChain {
-        let mut windows = self.windows.borrow_mut();
-
-        let window = windows
-            .get_mut(&window_id)
-            .expect("no window for redraw request ID");
-
-        if window.swap_chain.swap_chain.is_none() {
-            window.rebuild_swap_chain(window.tracked_state.physical_size.into());
-        }
-
-        window
-            .swap_chain
-            .swap_chain
-            .take()
-            .expect("missing swap chain")
-    }
-
     /// Return the **Rect** for the currently focused window.
     ///
     /// The **Rect** coords are described in "points" (pixels divided by the hidpi factor).
@@ -788,7 +783,7 @@ impl App {
     }
 
     /// Return the wgpu `Backends` in use.
-    pub fn backends(&self) -> wgpu::BackendBit {
+    pub fn backends(&self) -> wgpu::Backends {
         self.backends
     }
 
@@ -871,14 +866,6 @@ impl App {
         self.event_loop_proxy.clone()
     }
 
-    /// A builder for creating a new **Ui**.
-    ///
-    /// Each **Ui** is associated with one specific window. By default, this is the window returned
-    /// by `App::window_id` (the currently focused window).
-    pub fn new_ui(&self) -> ui::Builder {
-        ui::Builder::new(self)
-    }
-
     /// Produce the **App**'s **Draw** API for drawing geometry and text with colors and textures.
     ///
     /// **Note:** You can also create your own **Draw** instances via `Draw::new()`! This method
@@ -953,7 +940,7 @@ impl draw::Draw {
         let renderers = app.draw_state.renderers.borrow_mut();
         let renderer = RefMut::map(renderers, |renderers| {
             renderers.entry(window_id).or_insert_with(|| {
-                let device = window.swap_chain_device();
+                let device = window.device();
                 let frame_dims: [u32; 2] = window.tracked_state.physical_size.into();
                 let scale_factor = window.tracked_state.scale_factor as f32;
                 let msaa_samples = window.msaa_samples();
@@ -971,11 +958,20 @@ impl draw::Draw {
 
         let scale_factor = window.tracked_state.scale_factor as _;
         let mut renderer = renderer.borrow_mut();
-        renderer.render_to_frame(window.swap_chain_device(), self, scale_factor, frame);
+        renderer.render_to_frame(window.device(), self, scale_factor, frame);
         Ok(())
     }
 }
 
+impl<'a> wgpu::WithDeviceQueuePair for &'a crate::app::App {
+    fn with_device_queue_pair<F, O>(self, f: F) -> O
+    where
+        F: FnOnce(&wgpu::Device, &wgpu::Queue) -> O,
+    {
+        self.main_window().with_device_queue_pair(f)
+    }
+}
+
 /// Attempt to find the assets directory path relative to the executable location.
 pub fn find_assets_path() -> Result<PathBuf, find_folder::Error> {
     let exe_path = std::env::current_exe()?;
@@ -1077,6 +1073,7 @@ fn run_loop<M, E>(
         loop_start,
         last_update: loop_start,
         total_updates: 0,
+        events_since_wakeup: 0,
     };
 
     // Run the event loop.
@@ -1100,6 +1097,10 @@ fn run_loop<M, E>(
                     match loop_mode {
                         LoopMode::NTimes { number_of_updates }
                             if loop_state.total_updates >= number_of_updates as u64 => {}
+                        // Sometimes winit interrupts ControlFlow::Wait for no good reason, so we
+                        // make sure that there were some events in order to do an update when
+                        // LoopMode::Wait is used.
+                        LoopMode::Wait if loop_state.events_since_wakeup == 0 => {}
                         _ => do_update(&mut loop_state),
                     }
                 }
@@ -1110,24 +1111,21 @@ fn run_loop<M, E>(
             // TODO: Only request a frame from the user if this redraw was requested following an
             // update. Otherwise, just use the existing intermediary frame.
             winit::event::Event::RedrawRequested(window_id) => {
-                // Take the render data and swapchain.
-                // We'll replace them before the end of this block.
-                let (mut swap_chain, nth_frame) = {
-                    let swap_chain = app.take_or_rebuild_swap_chain(window_id);
-
-                    let mut windows = app.windows.borrow_mut();
-                    let window = windows
-                        .get_mut(&window_id)
-                        .expect("no window for redraw request ID");
-
-                    let nth_frame = window.frame_count;
-                    window.frame_count += 1;
-                    (swap_chain, nth_frame)
-                };
-
                 if let Some(model) = model.as_ref() {
-                    let mut swap_chain_output = swap_chain.get_current_frame();
-                    if let Err(e) = &swap_chain_output {
+                    // Retrieve the surface frame and the number of this frame.
+                    // NOTE: We avoid mutably borrowing `windows` map any longer than necessary to
+                    // avoid restricting users from accessing `windows` during `view`.
+                    let (mut surface_tex_result, nth_frame) = {
+                        let mut windows = app.windows.borrow_mut();
+                        let window = windows
+                            .get_mut(&window_id)
+                            .expect("no window for `RedrawRequest`");
+                        let texture = window.surface.get_current_texture();
+                        let nth_frame = window.frame_count;
+                        (texture, nth_frame)
+                    };
+
+                    if let Err(e) = &surface_tex_result {
                         match e {
                             // Sometimes redraws get delivered before resizes on x11 for unclear reasons.
                             // It goes all the way down to the API: if you ask x11 about the window size
@@ -1137,21 +1135,28 @@ fn run_loop<M, E>(
                             // If you turn on debug logging this does occasionally cause some vulkan
                             // validation errors... that's not great.
                             // TODO find a better long-term fix than ignoring.
-                            wgpu::SwapChainError::Lost => {
-                                // Attempt to rebuild the swap chain
-                                swap_chain = app.take_or_rebuild_swap_chain(window_id);
-                                swap_chain_output = swap_chain.get_current_frame();
+                            wgpu::SurfaceError::Lost => {
+                                // Attempt to reconfigure the surface.
+                                let mut windows = app.windows.borrow_mut();
+                                let window = windows
+                                    .get_mut(&window_id)
+                                    .expect("no window for `RedrawRequest`");
+                                window
+                                    .reconfigure_surface(window.tracked_state.physical_size.into());
+                                surface_tex_result = window.surface.get_current_texture();
                             }
-                            wgpu::SwapChainError::Outdated => {} // skip frame
-                            wgpu::SwapChainError::Timeout => {}  // skip frame
-                            wgpu::SwapChainError::OutOfMemory => {
-                                panic!("out of memory acquiring the swap chain frame: {}", e);
+                            wgpu::SurfaceError::Outdated => {} // skip frame
+                            wgpu::SurfaceError::Timeout => {}  // skip frame
+                            wgpu::SurfaceError::OutOfMemory => {
+                                panic!("out of memory acquiring the surface frame: {}", e);
                             }
                         }
                     }
 
-                    if let Ok(swap_chain_output) = swap_chain_output {
-                        let swap_chain_texture = &swap_chain_output.output.view;
+                    if let Ok(surface_tex) = surface_tex_result {
+                        let surface_texture = &surface_tex
+                            .texture
+                            .create_view(&wgpu::TextureViewDescriptor::default());
 
                         // Borrow the window now that we don't need it mutably until setting the render
                         // data back.
@@ -1170,14 +1175,21 @@ fn run_loop<M, E>(
                             .into();
                         let window_rect = geom::Rect::from_w_h(w, h);
                         let raw_frame = RawFrame::new_empty(
-                            window.swap_chain_device_queue_pair().clone(),
+                            window.device_queue_pair().clone(),
                             window_id,
                             nth_frame,
-                            swap_chain_texture,
-                            window.swap_chain.descriptor.format,
+                            surface_texture,
+                            window.surface_conf.format,
                             window_rect,
                         );
 
+                        // Clear the raw frame immediately once the window is invalidated
+                        if window.is_invalidated {
+                            if let Some(data) = frame_data {
+                                raw_frame.clear(&data.render.texture_view(), window.clear_color);
+                            }
+                        }
+
                         // If the user specified a view function specifically for this window, use it.
                         // Otherwise, use the fallback, default view passed to the app if there was one.
                         let window_view = window.user_functions.view.clone();
@@ -1221,16 +1233,21 @@ fn run_loop<M, E>(
                             },
                         }
 
+                        // Queue has been submitted by now, time to present.
+                        surface_tex.present();
+
                         // Release immutable lock
                         drop(windows);
 
-                        // Replace the render data and swap chain.
+                        // Increment the window's frame count.
                         let mut windows = app.windows.borrow_mut();
                         let window = windows
                             .get_mut(&window_id)
                             .expect("no window for redraw request ID");
 
-                        window.swap_chain.swap_chain = Some(swap_chain);
+                        // Assume invalidated window was cleared above before `view()`
+                        window.is_invalidated = false;
+                        window.frame_count += 1;
                     }
                 }
             }
@@ -1247,16 +1264,19 @@ fn run_loop<M, E>(
             }
 
             // Ignore wake-up events for now. Currently, these can only be triggered via the app proxy.
-            winit::event::Event::NewEvents(_) => {}
+            winit::event::Event::NewEvents(_) => {
+                loop_state.events_since_wakeup = 0;
+            }
 
             // Track the number of updates since the last I/O event.
             // This is necessary for the `Wait` loop mode to behave correctly.
             ref _other_event => {
                 loop_state.updates_since_event = 0;
+                loop_state.events_since_wakeup += 1;
             }
         }
 
-        // We must re-build the swap chain if the window was resized.
+        // We must reconfigure the wgpu surface if the window was resized.
         if let winit::event::Event::WindowEvent {
             ref mut event,
             window_id,
@@ -1266,7 +1286,7 @@ fn run_loop<M, E>(
                 winit::event::WindowEvent::Resized(new_inner_size) => {
                     let mut windows = app.windows.borrow_mut();
                     if let Some(window) = windows.get_mut(&window_id) {
-                        window.rebuild_swap_chain(new_inner_size.clone().into());
+                        window.reconfigure_surface(new_inner_size.clone().into());
                     }
                 }
 
@@ -1277,7 +1297,7 @@ fn run_loop<M, E>(
                     let mut windows = app.windows.borrow_mut();
                     if let Some(window) = windows.get_mut(&window_id) {
                         window.tracked_state.scale_factor = *scale_factor;
-                        window.rebuild_swap_chain(new_inner_size.clone().into());
+                        window.reconfigure_surface(new_inner_size.clone().into());
                     }
                 }
 
@@ -1536,20 +1556,6 @@ where
 
                 _ => (),
             }
-
-            // See if the event could be interpreted as a `ui::Input`. If so, submit it to the
-            // `Ui`s associated with this window.
-            if let Some(window) = app.windows.borrow().get(&window_id) {
-                if let Some(input) = ui::winit_window_event_to_input(event, window) {
-                    if let Some(handles) = app.ui.windows.borrow().get(&window_id) {
-                        for handle in handles {
-                            if let Some(ref tx) = handle.input_tx {
-                                tx.try_send(input.clone()).ok();
-                            }
-                        }
-                    }
-                }
-            }
         }
     }
 
@@ -1659,6 +1665,9 @@ where
             match simple {
                 event::WindowEvent::KeyPressed(key) => call_user_function!(key_pressed, key),
                 event::WindowEvent::KeyReleased(key) => call_user_function!(key_released, key),
+                event::WindowEvent::ReceivedCharacter(char) => {
+                    call_user_function!(received_character, char)
+                }
                 event::WindowEvent::MouseMoved(pos) => call_user_function!(mouse_moved, pos),
                 event::WindowEvent::MousePressed(button) => {
                     call_user_function!(mouse_pressed, button)
