@@ -3,6 +3,7 @@
 //! Create a new window via `app.new_window()`. This produces a [**Builder**](./struct.Builder.html)
 //! which can be used to build a [**Window**](./struct.Window.html).
 
+use crate::color::IntoLinSrgba;
 use crate::event::{
     Key, MouseButton, MouseScrollDelta, TouchEvent, TouchPhase, TouchpadPressure, WindowEvent,
 };
@@ -28,7 +29,7 @@ pub const DEFAULT_DIMENSIONS: LogicalSize<geom::scalar::Default> = LogicalSize {
     height: 768.0,
 };
 
-/// The default minimum dimensions used for the swap chain
+/// The default minimum dimensions used for the surface
 pub const MIN_SC_PIXELS: PhysicalSize<u32> = PhysicalSize {
     width: 2,
     height: 2,
@@ -39,13 +40,15 @@ pub struct Builder<'app> {
     app: &'app App,
     window: winit::window::WindowBuilder,
     title_was_set: bool,
-    swap_chain_builder: SwapChainBuilder,
+    surface_conf_builder: SurfaceConfigurationBuilder,
     power_preference: wgpu::PowerPreference,
+    force_fallback_adapter: bool,
     device_desc: Option<wgpu::DeviceDescriptor<'static>>,
     user_functions: UserFunctions,
     msaa_samples: Option<u32>,
     max_capture_frame_jobs: u32,
     capture_frame_timeout: Option<Duration>,
+    clear_color: Option<wgpu::Color>,
 }
 
 /// For storing all user functions within the window.
@@ -56,6 +59,7 @@ pub(crate) struct UserFunctions {
     pub(crate) raw_event: Option<RawEventFnAny>,
     pub(crate) key_pressed: Option<KeyPressedFnAny>,
     pub(crate) key_released: Option<KeyReleasedFnAny>,
+    pub(crate) received_character: Option<ReceivedCharacterFnAny>,
     pub(crate) mouse_moved: Option<MouseMovedFnAny>,
     pub(crate) mouse_pressed: Option<MousePressedFnAny>,
     pub(crate) mouse_released: Option<MouseReleasedFnAny>,
@@ -79,8 +83,8 @@ pub type ViewFn<Model> = fn(&App, &Model, Frame);
 
 /// The user function type for drawing their model to the surface of a single window.
 ///
-/// Unlike the `ViewFn`, the `RawViewFn` is designed for drawing directly to a window's swap chain
-/// images rather than to a convenient intermediary image.
+/// Unlike the `ViewFn`, the `RawViewFn` is designed for drawing directly to a window's surface
+/// texture rather than to a convenient intermediary image.
 pub type RawViewFn<Model> = fn(&App, &Model, RawFrame);
 
 /// The same as `ViewFn`, but provides no user model to draw from.
@@ -108,6 +112,9 @@ pub type KeyPressedFn<Model> = fn(&App, &mut Model, Key);
 /// A function for processing key release events.
 pub type KeyReleasedFn<Model> = fn(&App, &mut Model, Key);
 
+/// A function for processing received characters.
+pub type ReceivedCharacterFn<Model> = fn(&App, &mut Model, char);
+
 /// A function for processing mouse moved events.
 pub type MouseMovedFn<Model> = fn(&App, &mut Model, Point2);
 
@@ -206,6 +213,7 @@ fn_any!(EventFn<M>, EventFnAny);
 fn_any!(RawEventFn<M>, RawEventFnAny);
 fn_any!(KeyPressedFn<M>, KeyPressedFnAny);
 fn_any!(KeyReleasedFn<M>, KeyReleasedFnAny);
+fn_any!(ReceivedCharacterFn<M>, ReceivedCharacterFnAny);
 fn_any!(MouseMovedFn<M>, MouseMovedFnAny);
 fn_any!(MousePressedFn<M>, MousePressedFnAny);
 fn_any!(MouseReleasedFn<M>, MouseReleasedFnAny);
@@ -226,24 +234,26 @@ fn_any!(ClosedFn<M>, ClosedFnAny);
 /// A nannou window.
 ///
 /// The **Window** acts as a wrapper around the `winit::window::Window` and the `wgpu::Surface`
-/// types. It also manages the associated swap chain, providing a more nannou-friendly API.
+/// types.
 #[derive(Debug)]
 pub struct Window {
     pub(crate) window: winit::window::Window,
     pub(crate) surface: wgpu::Surface,
+    pub(crate) surface_conf: wgpu::SurfaceConfiguration,
     pub(crate) device_queue_pair: Arc<wgpu::DeviceQueuePair>,
     msaa_samples: u32,
-    pub(crate) swap_chain: WindowSwapChain,
     pub(crate) frame_data: Option<FrameData>,
     pub(crate) frame_count: u64,
     pub(crate) user_functions: UserFunctions,
     pub(crate) tracked_state: TrackedState,
+    pub(crate) is_invalidated: bool, // Whether framebuffer must be cleared
+    pub(crate) clear_color: wgpu::Color,
 }
 
-// Data related to `Frame`s produced for this window's swapchain textures.
+// Data related to `Frame`s produced for this window's surface textures.
 #[derive(Debug)]
 pub(crate) struct FrameData {
-    // Data for rendering a `Frame`'s intermediary image to a swap chain image.
+    // Data for rendering a `Frame`'s intermediary image to a surface texture.
     pub(crate) render: frame::RenderData,
     // Data for capturing a `Frame`'s intermediary image before submission.
     pub(crate) capture: frame::CaptureData,
@@ -260,59 +270,48 @@ pub(crate) struct TrackedState {
     pub(crate) physical_size: winit::dpi::PhysicalSize<u32>,
 }
 
-/// A swap_chain and its images associated with a single window.
-pub(crate) struct WindowSwapChain {
-    // The descriptor used to create the original swap chain. Useful for recreation.
-    pub(crate) descriptor: wgpu::SwapChainDescriptor,
-    // This is an `Option` in order to allow for separating ownership of the swapchain from the
-    // window during a `RedrawRequest`. Other than during `RedrawRequest`, this should always be
-    // `Some`.
-    pub(crate) swap_chain: Option<wgpu::SwapChain>,
-}
-
-/// SwapChain building parameters for which Nannou will provide a default if unspecified.
+/// Surface configuration for which nannou will provide a default if unspecified.
 ///
 /// See the builder methods for more details on each parameter.
 #[derive(Clone, Debug, Default)]
-pub struct SwapChainBuilder {
-    pub usage: Option<wgpu::TextureUsage>,
+pub struct SurfaceConfigurationBuilder {
+    pub usage: Option<wgpu::TextureUsages>,
     pub format: Option<wgpu::TextureFormat>,
     pub present_mode: Option<wgpu::PresentMode>,
 }
 
-impl SwapChainBuilder {
-    pub const DEFAULT_USAGE: wgpu::TextureUsage = wgpu::TextureUsage::RENDER_ATTACHMENT;
+impl SurfaceConfigurationBuilder {
+    /// Only used in the case that the `wgpu::Surface::get_preferred_format` method returns `None`.
     pub const DEFAULT_FORMAT: wgpu::TextureFormat = wgpu::TextureFormat::Bgra8UnormSrgb;
     pub const DEFAULT_PRESENT_MODE: wgpu::PresentMode = wgpu::PresentMode::Fifo;
+    pub const DEFAULT_USAGE: wgpu::TextureUsages = wgpu::TextureUsages::RENDER_ATTACHMENT;
 
-    /// A new empty **SwapChainBuilder** with all parameters set to `None`.
+    /// A new empty **SurfaceConfigurationBuilder** with all parameters set to `None`.
     pub fn new() -> Self {
         Default::default()
     }
 
-    /// Create a **SwapChainBuilder** from an existing descriptor.
-    ///
-    /// The resulting swap chain parameters will match that of the given `SwapChainDescriptor`.
-    pub fn from_descriptor(desc: &wgpu::SwapChainDescriptor) -> Self {
-        SwapChainBuilder::new()
-            .usage(desc.usage)
-            .format(desc.format)
-            .present_mode(desc.present_mode)
+    /// Create a **SurfaceConfigurationBuilder** from an existing descriptor.
+    pub fn from_configuration(conf: &wgpu::SurfaceConfiguration) -> Self {
+        SurfaceConfigurationBuilder::new()
+            .usage(conf.usage)
+            .format(conf.format)
+            .present_mode(conf.present_mode)
     }
 
-    /// Specify the texture usage for the swap chain.
-    pub fn usage(mut self, usage: wgpu::TextureUsage) -> Self {
+    /// Specify the texture usages for the surface.
+    pub fn usage(mut self, usage: wgpu::TextureUsages) -> Self {
         self.usage = Some(usage);
         self
     }
 
-    /// Specify the texture format for the swap chain.
+    /// Specify the texture format for the surface.
     pub fn format(mut self, format: wgpu::TextureFormat) -> Self {
         self.format = Some(format);
         self
     }
 
-    /// The way in which swap chain images are presented to the display.
+    /// The way in which a surface's frames are presented to the display.
     ///
     /// By default, nannou will attempt to select the ideal present mode depending on the current
     /// app `LoopMode`.
@@ -321,31 +320,34 @@ impl SwapChainBuilder {
         self
     }
 
-    /// Build the swap chain.
+    /// Build the surface configuration.
     pub(crate) fn build(
         self,
-        device: &wgpu::Device,
         surface: &wgpu::Surface,
+        adapter: &wgpu::Adapter,
         [width_px, height_px]: [u32; 2],
-    ) -> (wgpu::SwapChain, wgpu::SwapChainDescriptor) {
+    ) -> wgpu::SurfaceConfiguration {
         let usage = self.usage.unwrap_or(Self::DEFAULT_USAGE);
-        let format = self.format.unwrap_or(Self::DEFAULT_FORMAT);
+        let format = self
+            .format
+            .or_else(|| surface.get_preferred_format(&adapter))
+            .unwrap_or(Self::DEFAULT_FORMAT);
         let present_mode = self.present_mode.unwrap_or(Self::DEFAULT_PRESENT_MODE);
-        let desc = wgpu::SwapChainDescriptor {
+        wgpu::SurfaceConfiguration {
             usage,
             format,
             width: width_px,
             height: height_px,
             present_mode,
-        };
-        let swap_chain = device.create_swap_chain(surface, &desc);
-        (swap_chain, desc)
+        }
     }
 }
 
 impl<'app> Builder<'app> {
     /// The default power preference used to request the WGPU adapter.
     pub const DEFAULT_POWER_PREFERENCE: wgpu::PowerPreference = wgpu::DEFAULT_POWER_PREFERENCE;
+    /// The default `force_fallback_adapter` field used to request the WGPU adapter.
+    pub const DEFAULT_FORCE_FALLBACK_ADAPTER: bool = false;
 
     /// Begin building a new window.
     pub fn new(app: &'app App) -> Self {
@@ -353,13 +355,15 @@ impl<'app> Builder<'app> {
             app,
             window: winit::window::WindowBuilder::new(),
             title_was_set: false,
-            swap_chain_builder: Default::default(),
+            surface_conf_builder: Default::default(),
             power_preference: Self::DEFAULT_POWER_PREFERENCE,
+            force_fallback_adapter: Self::DEFAULT_FORCE_FALLBACK_ADAPTER,
             device_desc: None,
             user_functions: Default::default(),
             msaa_samples: None,
             max_capture_frame_jobs: Default::default(),
             capture_frame_timeout: Default::default(),
+            clear_color: None,
         }
     }
 
@@ -369,9 +373,12 @@ impl<'app> Builder<'app> {
         self
     }
 
-    /// Specify a set of parameters for building the window surface swap chain.
-    pub fn swap_chain_builder(mut self, swap_chain_builder: SwapChainBuilder) -> Self {
-        self.swap_chain_builder = swap_chain_builder;
+    /// Specify a set of parameters for building the window surface.
+    pub fn surface_conf_builder(
+        mut self,
+        surface_conf_builder: SurfaceConfigurationBuilder,
+    ) -> Self {
+        self.surface_conf_builder = surface_conf_builder;
         self
     }
 
@@ -383,6 +390,15 @@ impl<'app> Builder<'app> {
         self
     }
 
+    /// Indicates that only a fallback adapter can be returned. This is generally a "software"
+    /// implementation on the system..
+    ///
+    /// By default, this is `false`.
+    pub fn force_fallback_adapter(mut self, force: bool) -> Self {
+        self.force_fallback_adapter = force;
+        self
+    }
+
     /// Specify a device descriptor to use when requesting the logical device from the adapter.
     /// This allows for specifying custom wgpu device extensions.
     pub fn device_descriptor(mut self, device_desc: wgpu::DeviceDescriptor<'static>) -> Self {
@@ -397,14 +413,14 @@ impl<'app> Builder<'app> {
     ///
     /// **Note:** This parameter has no meaning if the window uses a **raw_view** function for
     /// rendering graphics to the window rather than a **view** function. This is because the
-    /// **raw_view** function provides a **RawFrame** with direct access to the swap chain image
+    /// **raw_view** function provides a **RawFrame** with direct access to the surface texture
     /// itself and thus must manage their own MSAA pass.
     ///
     /// On the other hand, the `view` function provides the `Frame` type which allows the user to
     /// render to a multisampled intermediary image allowing Nannou to take care of resolving the
-    /// multisampled image to the swap chain image. In order to avoid confusion, The `Window::build`
-    /// method will `panic!` if the user tries to specify `msaa_samples` as well as a `raw_view`
-    /// method.
+    /// multisampled texture to the surface texture. In order to avoid confusion, The
+    /// `Window::build` method will `panic!` if the user tries to specify `msaa_samples` as well as
+    /// a `raw_view` method.
     ///
     /// *TODO: Perhaps it would be worth adding two separate methods for specifying msaa samples.
     /// One for forcing a certain number of samples and returning an error otherwise, and another
@@ -438,7 +454,7 @@ impl<'app> Builder<'app> {
     /// surface of the window on your display.
     ///
     /// Unlike the **ViewFn**, the **RawViewFn** provides a **RawFrame** that is designed for
-    /// drawing directly to a window's swap chain images, rather than to a convenient intermediary
+    /// drawing directly to a window's surface texture, rather than to a convenient intermediary
     /// image.
     pub fn raw_view<M>(mut self, raw_view_fn: RawViewFn<M>) -> Self
     where
@@ -448,6 +464,20 @@ impl<'app> Builder<'app> {
         self
     }
 
+    /// Set the initial color of the window background
+    /// when its contents are invalidated, e.g. upon window resize.
+    pub fn clear_color<C>(mut self, color: C) -> Self
+    where
+        C: IntoLinSrgba<f32>,
+    {
+        let lin_srgba = color.into_lin_srgba();
+        let (r, g, b, a) = lin_srgba.into_components();
+        let (r, g, b, a) = (r as f64, g as f64, b as f64, a as f64);
+
+        self.clear_color = Some(wgpu::Color { r, g, b, a });
+        self
+    }
+
     /// A function for updating your model on `WindowEvent`s associated with this window.
     ///
     /// These include events such as key presses, mouse movement, clicks, resizing, etc.
@@ -508,6 +538,14 @@ impl<'app> Builder<'app> {
         self
     }
 
+    pub fn received_character<M>(mut self, f: ReceivedCharacterFn<M>) -> Self
+    where
+        M: 'static,
+    {
+        self.user_functions.received_character = Some(ReceivedCharacterFnAny::from_fn_ptr(f));
+        self
+    }
+
     /// A function for processing mouse moved events associated with this window.
     pub fn mouse_moved<M>(mut self, f: MouseMovedFn<M>) -> Self
     where
@@ -694,13 +732,15 @@ impl<'app> Builder<'app> {
             app,
             mut window,
             title_was_set,
-            swap_chain_builder,
+            surface_conf_builder,
             power_preference,
+            force_fallback_adapter,
             device_desc,
             user_functions,
             msaa_samples,
             max_capture_frame_jobs,
             capture_frame_timeout,
+            clear_color,
         } = self;
 
         // If the title was not set, default to the "nannou - <exe_name>".
@@ -773,17 +813,26 @@ impl<'app> Builder<'app> {
                 dim.into()
             });
 
-        // Use the `initial_swapchain_dimensions` as the default dimensions for the window if none
+        // Use the `initial_window_size` as the default dimensions for the window if none
         // were specified.
         if window.window.inner_size.is_none() && window.window.fullscreen.is_none() {
             window.window.inner_size = Some(initial_window_size);
         }
 
-        // Set a default minimum window size to prevent requesting a too-small swap chains
+        // Set a default minimum window size for configuring the surface.
         if window.window.min_inner_size.is_none() && window.window.fullscreen.is_none() {
             window.window.min_inner_size = Some(winit::dpi::Size::Physical(MIN_SC_PIXELS));
         }
 
+        // Background must be initially cleared
+        let is_invalidated = true;
+
+        let clear_color = clear_color.unwrap_or_else(|| {
+            let mut color: wgpu::Color = Default::default();
+            color.a = if window.window.transparent { 0.0 } else { 1.0 };
+            color
+        });
+
         // Build the window.
         let window = {
             let window_target = app
@@ -801,6 +850,7 @@ impl<'app> Builder<'app> {
         let request_adapter_opts = wgpu::RequestAdapterOptions {
             power_preference,
             compatible_surface: Some(&surface),
+            force_fallback_adapter,
         };
         let adapter = app
             .wgpu_adapters()
@@ -811,24 +861,24 @@ impl<'app> Builder<'app> {
         let device_desc = device_desc.unwrap_or_else(wgpu::default_device_descriptor);
         let device_queue_pair = adapter.get_or_request_device(device_desc);
 
-        // Build the swapchain.
+        // Configure the surface.
         let win_physical_size = window.inner_size();
         let win_dims_px: [u32; 2] = win_physical_size.into();
         let device = device_queue_pair.device();
-        let (swap_chain, swap_chain_desc) =
-            swap_chain_builder.build(&device, &surface, win_dims_px);
+        let surface_conf = surface_conf_builder.build(&surface, &*adapter, win_dims_px);
+        surface.configure(&device, &surface_conf);
 
-        // If we're using an intermediary image for rendering frames to swap_chain images, create
+        // If we're using an intermediary image for rendering frames to surface textures, create
         // the necessary render data.
         let (frame_data, msaa_samples) = match user_functions.view {
             Some(View::WithModel(_)) | Some(View::Sketch(_)) | None => {
                 let msaa_samples = msaa_samples.unwrap_or(Frame::DEFAULT_MSAA_SAMPLES);
                 // TODO: Verity that requested sample count is valid for surface?
-                let swap_chain_dims = [swap_chain_desc.width, swap_chain_desc.height];
+                let surface_dims = [surface_conf.width, surface_conf.height];
                 let render = frame::RenderData::new(
                     &device,
-                    swap_chain_dims,
-                    swap_chain_desc.format,
+                    surface_dims,
+                    surface_conf.format,
                     msaa_samples,
                 );
                 let capture =
@@ -841,10 +891,6 @@ impl<'app> Builder<'app> {
 
         let window_id = window.id();
         let frame_count = 0;
-        let swap_chain = WindowSwapChain {
-            descriptor: swap_chain_desc,
-            swap_chain: Some(swap_chain),
-        };
 
         let tracked_state = TrackedState {
             scale_factor: window.scale_factor(),
@@ -854,13 +900,15 @@ impl<'app> Builder<'app> {
         let window = Window {
             window,
             surface,
+            surface_conf,
             device_queue_pair,
             msaa_samples,
-            swap_chain,
             frame_data,
             frame_count,
             user_functions,
             tracked_state,
+            is_invalidated,
+            clear_color,
         };
         app.windows.borrow_mut().insert(window_id, window);
 
@@ -882,11 +930,13 @@ impl<'app> Builder<'app> {
             title_was_set,
             device_desc,
             power_preference,
-            swap_chain_builder,
+            force_fallback_adapter,
+            surface_conf_builder,
             user_functions,
             msaa_samples,
             max_capture_frame_jobs,
             capture_frame_timeout,
+            clear_color,
         } = self;
         let window = map(window);
         Builder {
@@ -895,11 +945,13 @@ impl<'app> Builder<'app> {
             title_was_set,
             device_desc,
             power_preference,
-            swap_chain_builder,
+            force_fallback_adapter,
+            surface_conf_builder,
             user_functions,
             msaa_samples,
             max_capture_frame_jobs,
             capture_frame_timeout,
+            clear_color,
         }
     }
 
@@ -1321,35 +1373,30 @@ impl Window {
 
     // Access to wgpu API.
 
-    /// Returns a reference to the window's wgpu swap chain surface.
+    /// Returns a reference to the window's wgpu surface.
     pub fn surface(&self) -> &wgpu::Surface {
         &self.surface
     }
 
-    /// The descriptor for the swap chain associated with this window's wgpu surface.
-    pub fn swap_chain_descriptor(&self) -> &wgpu::SwapChainDescriptor {
-        &self.swap_chain.descriptor
+    /// The current configuration of the window's wgpu surface.
+    pub fn surface_configuration(&self) -> &wgpu::SurfaceConfiguration {
+        &self.surface_conf
     }
 
-    /// The wgpu logical device on which the window's swap chain is running.
-    ///
-    /// This is shorthand for `DeviceOwned::device(window.swap_chain())`.
-    pub fn swap_chain_device(&self) -> &wgpu::Device {
+    /// The wgpu logical device on which the window's wgpu surface is running.
+    pub fn device(&self) -> &wgpu::Device {
         self.device_queue_pair.device()
     }
 
-    /// The wgpu graphics queue on which the window swap chain work is run.
-    ///
-    /// The queue is guarded by a `Mutex` in order to synchronise submissions of command buffers in
-    /// cases that the queue is shared between more than one window.
-    pub fn swap_chain_queue(&self) -> &wgpu::Queue {
+    /// The wgpu graphics queue to which the window's wgpu surface frames are submitted.
+    pub fn queue(&self) -> &wgpu::Queue {
         self.device_queue_pair.queue()
     }
 
     /// Provides access to the device queue pair and the `Arc` behind which it is stored. This can
-    /// be useful in cases where using references provided by the `swap_chain_device` or
-    /// `swap_chain_queue` methods cause awkward ownership problems.
-    pub fn swap_chain_device_queue_pair(&self) -> &Arc<wgpu::DeviceQueuePair> {
+    /// be useful in cases where using references provided by the `device` or `queue` methods cause
+    /// awkward ownership problems.
+    pub fn device_queue_pair(&self) -> &Arc<wgpu::DeviceQueuePair> {
         &self.device_queue_pair
     }
 
@@ -1364,34 +1411,29 @@ impl Window {
 
     // Custom methods.
 
-    // A utility function to simplify the recreation of a swap_chain.
-    // Syncs the window tracked size with the swap chain size.
-    pub(crate) fn rebuild_swap_chain(&mut self, size_px: [u32; 2]) {
-        if self.swap_chain.swap_chain.is_some() {
-            std::mem::drop(self.swap_chain.swap_chain.take());
-        }
-        let [width, height] = size_px;
-
-        // Ensure physical window size matches swap chain size
-        // Prevents crashing if a too-small swap chain is created
-        self.tracked_state.physical_size.width = width.max(MIN_SC_PIXELS.width);
-        self.tracked_state.physical_size.height = height.max(MIN_SC_PIXELS.height);
-
-        self.swap_chain.descriptor.width = self.tracked_state.physical_size.width;
-        self.swap_chain.descriptor.height = self.tracked_state.physical_size.height;
-        self.swap_chain.swap_chain = Some(
-            self.swap_chain_device()
-                .create_swap_chain(&self.surface, &self.swap_chain.descriptor),
-        );
+    // A utility function to simplify the reconfiguration of the window's wgpu surface.
+    //
+    // Upon resizing of the window, the window's surface needs to be reconfigured to match.
+    //
+    // Also syncs the window tracked size with the surface size.
+    pub(crate) fn reconfigure_surface(&mut self, [w_px, h_px]: [u32; 2]) {
+        self.tracked_state.physical_size.width = w_px.max(MIN_SC_PIXELS.width);
+        self.tracked_state.physical_size.height = h_px.max(MIN_SC_PIXELS.height);
+        self.surface_conf.width = self.tracked_state.physical_size.width;
+        self.surface_conf.height = self.tracked_state.physical_size.height;
+        self.surface.configure(self.device(), &self.surface_conf);
         if self.frame_data.is_some() {
             let render_data = frame::RenderData::new(
-                self.swap_chain_device(),
+                self.device(),
                 self.tracked_state.physical_size.into(),
-                self.swap_chain.descriptor.format,
+                self.surface_conf.format,
                 self.msaa_samples,
             );
             self.frame_data.as_mut().unwrap().render = render_data;
         }
+
+        // May contain uninitialized or previous contents, so must be cleared
+        self.is_invalidated = true;
     }
 
     /// Attempts to determine whether or not the window is currently fullscreen.
@@ -1429,6 +1471,13 @@ impl Window {
         self.capture_frame_inner(path.as_ref());
     }
 
+    /// Produces a reference to the inner winit window.
+    ///
+    /// This is sometimes useful for integration with other winit-aware libraries (e.g. UI).
+    pub fn winit_window(&self) -> &winit::window::Window {
+        &self.window
+    }
+
     fn capture_frame_inner(&self, path: &Path) {
         // If the parent directory does not exist, create it.
         let dir = path.parent().expect("capture_frame path has no directory");
@@ -1456,7 +1505,7 @@ impl Window {
     ) -> Result<(), wgpu::TextureCapturerAwaitWorkerTimeout<()>> {
         if let Some(frame_data) = self.frame_data.as_ref() {
             let capture_data = &frame_data.capture;
-            let device = self.swap_chain_device();
+            let device = self.device();
             return capture_data.texture_capturer.await_active_snapshots(device);
         }
         Ok(())
@@ -1487,18 +1536,6 @@ impl fmt::Debug for View {
     }
 }
 
-// Deref implementations.
-
-impl fmt::Debug for WindowSwapChain {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(
-            f,
-            "WindowSwapChain ( descriptor: {:?}, swap_chain: {:?} )",
-            self.descriptor, self.swap_chain,
-        )
-    }
-}
-
 // Error implementations.
 
 impl fmt::Display for BuildError {
@@ -1515,3 +1552,14 @@ impl From<winit::error::OsError> for BuildError {
         BuildError::WinitOsError(e)
     }
 }
+
+// Some WGPU helper implementations.
+
+impl<'a> wgpu::WithDeviceQueuePair for &'a crate::window::Window {
+    fn with_device_queue_pair<F, O>(self, f: F) -> O
+    where
+        F: FnOnce(&wgpu::Device, &wgpu::Queue) -> O,
+    {
+        self.device_queue_pair().with_device_queue_pair(f)
+    }
+}
